'' =================================================================================================
''
''   File....... jm_dualmotor_ez.spin
''   Purpose.... Simplified dual motor PWM control
''   Author..... Jon "JonnyMac" McPhalen (aka Jon Williams)
''               Copyright (c) 2011 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 15 MAY 2011
''
'' =================================================================================================


con

  #0, M_FWD, M_REV                                              ' direction control
  

var

  long  mxpwm[2]                                                ' pwm pins
  long  mxdir[2]                                                ' direction pins
  long  mxtix[2]                                                ' speed (duty cycle) tix

  long  pwmtix                                                  ' tix for period
  long  cog                                                     ' pwm cog id (1 - 8)

  long  stack[16]                                               ' stack for pwm cog                                               


pub start(m0p, m0d, m1p, m1d, freq) | ok

'' Start motor driver

  stop

  if (m0p => 0)                                                 ' m0 used?
    mxpwm[0] := m0p                                             '  set pwm pin
    mxdir[0] := m0d                                             '  set direction pin
    dira[m0d] := 1                                              '  make direction pin output    

  if (m1p => 0)    
    mxpwm[1] := m1p
    mxdir[1] := m1d
    dira[m1d] := 1 

  pwmtix := clkfreq / freq                                      ' ticks per pwm period

  ok := cog := cognew(dualpwm, @stack) + 1                      ' launch the PWM cog

  return cog   

  
pub stop | idx
  
'' Stops motor driver (if running)

  if cog
    cogstop(cog~ - 1)                                           ' stop motor cog
    repeat idx from 0 to 1
      if (mxpwm[idx] => 0)                                      ' if motor used
        dira[mxdir[idx]] := 0                                   '  release direction pin

  longfill(@mxpwm, -1, 2)                                       ' undefine PWM pins                           
   

pub setspeed(midx, speed)

'' Set speed counts and direction control for desired motor
'' -- midx  = motor index, 0 or 1
'' -- speed = motor speed expressed in percent, negative for reverse

  if ((midx <> 0) and (midx <> 1))
    return                                                      ' escape if bad index

  speed := -100 #> speed <# 100

  case speed
    -100..-1:                                                   ' reverse
      if (mxpwm[midx] => 0)                                     ' if used
        mxtix[midx] := pwmtix * (100 + speed) / 100             ' set reverse speed
        outa[mxdir[midx]] := M_REV                              ' set direction control
  
    0:                                                          ' stop
      if (mxpwm[midx] => 0)                                     
        mxtix[midx] := 0                                        
        outa[mxdir[midx]] := M_FWD

    1..100:                                                     ' forward
      if (mxpwm[midx] => 0)
        mxtix[midx] := pwmtix * speed / 100  
        outa[mxdir[midx]] := M_FWD
        

pri dualpwm | sync                                              ' launch with cognew

'' Controls one or two DC motors
'' -- requires h-bridge

  if (mxpwm[0] => 0)
    ctra := (%00100 << 26) | mxpwm[0]                           ' PWM / NCO on pin
    frqa := 1                                                   ' setup freq for pwm
    dira[mxpwm[0]] := 1                                         ' make output
  else
    frqa := 0                                                   ' don't use this counter
    phsa := 0

  if (mxpwm[1] => 0)
    ctrb := (%00100 << 26) | mxpwm[1]                      
    frqb := 1                                              
    dira[mxpwm[1]] := 1                                    
  else
    frqb := 0
    phsb := 0          

  sync := cnt                                                   ' sync timing
  repeat                                                        ' run until cog stopped
    if (mxpwm[0] => 0)                                          ' if motor used
      phsa := -mxtix[0]                                         '  set speed
    if (mxpwm[1] => 0)                                          
      phsb := -mxtix[1]                                    
    waitcnt(sync += pwmtix)                                     ' let period expire

     
dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}